@@ -1,11 +1,12 @@
-﻿from captcha.conf import settings
-from captcha.models import CaptchaStore, get_safe_now
+from captcha.conf import settings
+from captcha.models import CaptchaStore
 from django.core.exceptions import ImproperlyConfigured
 from django.core.urlresolvers import reverse, NoReverseMatch
 from django.forms import ValidationError
 from django.forms.fields import CharField, MultiValueField
 from django.forms.widgets import TextInput, MultiWidget, HiddenInput
 from django.utils.translation import ugettext_lazy
+from django.utils import timezone
 from django.template.loader import render_to_string
 from django.utils.safestring import mark_safe
 from six import u
@@ -27,7 +28,7 @@ class BaseCaptchaTextInput(MultiWidget):
             return value.split(',')
         return [None, None]
 
-    def fetch_captcha_store(self, name, value, attrs=None):
+    def fetch_captcha_store(self, name, value, attrs=None, generator=None):
         """
         Fetches a new CaptchaStore
         This has to be called inside render
@@ -37,7 +38,10 @@ class BaseCaptchaTextInput(MultiWidget):
         except NoReverseMatch:
             raise ImproperlyConfigured('Make sure you\'ve included captcha.urls as explained in the INSTALLATION section on http://readthedocs.org/docs/django-simple-captcha/en/latest/usage.html#installation')
 
-        key = CaptchaStore.generate_key()
+        if settings.CAPTCHA_GET_FROM_POOL:
+            key = CaptchaStore.pick()
+        else:
+            key = CaptchaStore.generate_key(generator)
 
         # these can be used by format_output and render
         self._value = [key, u('')]
@@ -64,7 +68,7 @@ class CaptchaTextInput(BaseCaptchaTextInput):
         self._args = kwargs
         self._args['output_format'] = self._args.get('output_format') or settings.CAPTCHA_OUTPUT_FORMAT
         self._args['field_template'] = self._args.get('field_template') or settings.CAPTCHA_FIELD_TEMPLATE
-        self._args['id_prefix'] = self._args.get('id_prefix')
+        # self._args['id_prefix'] = self._args.get('id_prefix')
 
         if self._args['output_format'] is None and self._args['field_template'] is None:
             raise ImproperlyConfigured('You MUST define either CAPTCHA_FIELD_TEMPLATE or CAPTCHA_OUTPUT_FORMAT setting. Please refer to http://readthedocs.org/docs/django-simple-captcha/en/latest/usage.html#installation')
@@ -79,8 +83,8 @@ class CaptchaTextInput(BaseCaptchaTextInput):
 
         super(CaptchaTextInput, self).__init__(attrs)
 
-    def build_attrs(self, extra_attrs=None, **kwargs):
-        ret = super(CaptchaTextInput, self).build_attrs(extra_attrs, **kwargs)
+    def build_attrs(self, *args, **kwargs):
+        ret = super(CaptchaTextInput, self).build_attrs(*args, **kwargs)
         if self._args.get('id_prefix') and 'id' in ret:
             ret['id'] = '%s_%s' % (self._args.get('id_prefix'), ret['id'])
         return ret
@@ -92,14 +96,14 @@ class CaptchaTextInput(BaseCaptchaTextInput):
         return ret
 
     def format_output(self, rendered_widgets):
-        hidden_field, text_field = rendered_widgets
-
+        # hidden_field, text_field = rendered_widgets
         if self._args['output_format']:
-            return self._args['output_format'] % {
+            ret = self._args['output_format'] % {
                 'image': self.image_and_audio,
                 'hidden_field': self.hidden_field,
                 'text_field': self.text_field
             }
+            return ret
 
         elif self._args['field_template']:
             context = {
@@ -110,7 +114,7 @@ class CaptchaTextInput(BaseCaptchaTextInput):
             return render_to_string(settings.CAPTCHA_FIELD_TEMPLATE, context)
 
     def render(self, name, value, attrs=None):
-        self.fetch_captcha_store(name, value, attrs)
+        self.fetch_captcha_store(name, value, attrs, self._args.get('generator'))
 
         context = {
             'image': self.image_url(),
@@ -127,6 +131,9 @@ class CaptchaTextInput(BaseCaptchaTextInput):
 
         return super(CaptchaTextInput, self).render(name, self._value, attrs=attrs)
 
+    def _render(self, template_name, context, renderer=None):
+        return self.format_output(None)
+
 
 class CaptchaField(MultiValueField):
     def __init__(self, *args, **kwargs):
@@ -141,7 +148,8 @@ class CaptchaField(MultiValueField):
 
         kwargs['widget'] = kwargs.pop('widget', CaptchaTextInput(
             output_format=kwargs.pop('output_format', None),
-            id_prefix=kwargs.pop('id_prefix', None)
+            id_prefix=kwargs.pop('id_prefix', None),
+            generator=kwargs.pop('generator', None)
         ))
 
         super(CaptchaField, self).__init__(fields, *args, **kwargs)
@@ -154,7 +162,8 @@ class CaptchaField(MultiValueField):
     def clean(self, value):
         super(CaptchaField, self).clean(value)
         response, value[1] = (value[1] or '').strip().lower(), ''
-        CaptchaStore.remove_expired()
+        if not settings.CAPTCHA_GET_FROM_POOL:
+            CaptchaStore.remove_expired()
         if settings.CAPTCHA_TEST_MODE and response.lower() == 'passed':
             # automatically pass the test
             try:
@@ -167,7 +176,7 @@ class CaptchaField(MultiValueField):
             pass
         else:
             try:
-                CaptchaStore.objects.get(response=response, hashkey=value[0], expiration__gt=get_safe_now()).delete()
+                CaptchaStore.objects.get(response=response, hashkey=value[0], expiration__gt=timezone.now()).delete()
             except CaptchaStore.DoesNotExist:
                 raise ValidationError(getattr(self, 'error_messages', {}).get('invalid', ugettext_lazy('Invalid CAPTCHA')))
         return value
