@@ -34,7 +34,7 @@ from_top = 4
 
 def getsize(font, text):
     if hasattr(font, 'getoffset'):
-        return [x + y for x, y in zip(font.getsize(text), font.getoffset(text))]
+        return tuple([x + y for x, y in zip(font.getsize(text), font.getoffset(text))])
     else:
         return font.getsize(text)
 
@@ -162,7 +162,7 @@ def captcha_refresh(request):
     if not request.is_ajax():
         raise Http404
 
-    new_key = CaptchaStore.generate_key()
+    new_key = CaptchaStore.pick()
     to_json_response = {
         'key': new_key,
         'image_url': captcha_image_url(new_key),
